	#tag Class
	Protected Class Client
	Inherits HTTPSecureSocket
		#tag Event
			Sub DownloadComplete(url as string, httpStatus as integer, headers as internetHeaders, file as folderItem)
			  // hide
			End Sub
		#tag EndEvent

		#tag Event
			Sub Error(code as integer)
			  dim b as boolean
			  select case code
			  case socketcore.opendrivererror
			    b = me.issueerror(kError_SocketError,"Socket drivers could not be initialized")
			  case socketcore.lostconnection
			    b = me.issueerror(kError_ConnectionLost,"Server connection has been lost")
			  case socketcore.nameresolutionerror
			    b = me.issueerror(kError_ServerNotFound,"Could not find server at specified location")
			  case socketcore.addressinuseerror
			    b = me.issueerror(kError_SocketError,"Cannot bind to port: in use")
			  case socketcore.invalidstateerror
			    b = me.issueerror(kError_SocketError,"Socket was left in invalid state")
			  case socketcore.invalidporterror
			    b = me.issueerror(kError_SocketError,"Cannot bind to port: permission denied")
			  case socketcore.outofmemoryerror
			    b = me.issueerror(kError_SocketError,"Out of memory")
			  else
			    b = me.issueerror(kError_SocketError,"Unknown socket error #" + str(code))
			  end
			End Sub
		#tag EndEvent

		#tag Event
			Sub PageReceived(url as string, httpStatus as integer, headers as internetHeaders, content as string)
			  SendComplete lastmessage
			  
			  
			  select case httpstatus
			  case 200 // success
			    dim xml as string = content
			    dim response as variant = ProcessResponse(xml)
			    if response <> nil then
			      ResponseReceived self.lastmessage.methodname,response
			    end
			  case 301,302 // moved
			    dim s as string = headers.value("Location")
			    if s <> "" then
			      self.post(s)
			    end
			  else
			    if me.issueerror(kError_HTTPError,"Connection did not complete due to HTTP error #" + str(httpstatus)) then
			    end
			  end
			End Sub
		#tag EndEvent

		#tag Event
			Sub ReceiveProgress(bytesReceived as integer, totalBytes as integer, newData as string)
			  ResponseReceiveProgress self.lastmessage,bytesreceived,totalbytes
			End Sub
		#tag EndEvent

		#tag Event
			Function SendProgress(BytesSent As Integer, BytesLeft As Integer) As Boolean
			  SendProgress self.lastmessage,bytessent,bytessent + bytesleft
			End Function
		#tag EndEvent

		#tag Event
			Sub HeadersReceived(headers as internetHeaders, httpStatus as integer)
			  //
			End Sub
		#tag EndEvent


		#tag Method, Flags = &h0
			Sub Constructor()
			  // Calling the overridden superclass constructor.
			  // Note that this may need modifications if there are multiple constructor choices.
			  // Possible constructor calls:
			  // Constructor() -- From HTTPSecureSocket
			  // Constructor() -- From TCPSocket
			  // Constructor() -- From SocketCore
			  Super.Constructor
			  Open
			End Sub
		#tag EndMethod

		#tag Method, Flags = &h21
			Private Function DefaultUserAgent() As String
			  dim s as string
			  s = "The ZAZ Studios XML-RPC Client " + me.Version + " (REALbasic "
			  s = s + RBVersionString + "; "
			  #if DebugBuild
			    s = s + "*Debug Build* "
			  #endif
			  #if TargetMachO
			    s = s + "Mac OS Carbon Mach-O"
			  #elseif TargetCarbon
			    s = s + "Mac OS Carbon PEF"
			  #elseif TargetMacOSClassic
			    s = s + "Mac OS Classic"
			  #elseif TargetLinux
			    s = s + "Linux"
			  #elseif TargetWin32
			    s = s + "Windows"
			  #endif
			  s = s + "; " + App.LongVersion + ")"
			  return s
			End Function
		#tag EndMethod

		#tag Method, Flags = &h0
			Function LastError() As Integer
			  return pLastError
			End Function
		#tag EndMethod

		#tag Method, Flags = &h0
			Function LastErrorString() As String
			  return pLastErrorString
			End Function
		#tag EndMethod

		#tag Method, Flags = &h21
			Private Function IssueError(ErrorCode As Integer, ErrorMsg As String = "") As Boolean
			  plasterror = errorcode
			  plasterrorstring = errormsg
			  return not Error(ErrorCode,ErrorMsg)
			End Function
		#tag EndMethod

		#tag Method, Flags = &h21
			Private Function ProcessResponse(XML As String) As Variant
			  dim xmldoc as xmldocument
			  dim firstelement,struct,codeNode,stringNode as xmlnode
			  dim response as variant
			  
			  if xml = "" then
			    if me.issueerror(me.kError_NoResponse) then
			      return nil
			    end
			  end
			  
			  xmldoc = new xmldocument(xml)
			  if xmldoc = nil then
			    if me.issueerror(me.kError_BadXML) then
			      return nil
			    end
			  end
			  
			  if xmldoc.documentelement.name <> "methodResponse" then
			    if me.issueerror(me.kError_BadXML) then
			      return nil
			    end
			  end
			  
			  firstelement = xmldoc.documentelement.firstchild
			  if firstelement.name = "params" then
			    response = decode(firstelement.child(0).child(0).child(0))
			  else
			    struct = firstelement.child(0).child(0)
			    codeNode = struct.child(0).child(1).child(0).child(0)
			    stringNode = struct.child(1).child(1).child(0).child(0)
			    
			    if me.issueerror(val(codenode.value),stringnode.value) then
			    end
			    return nil
			  end
			  
			  return response
			exception err as xmlexception
			  if me.issueerror(me.kError_BadXML) then
			    return nil
			  end
			End Function
		#tag EndMethod

		#tag Method, Flags = &h0
			Sub SendMessage(TheMessage As Message)
			  dim agent as string = useragent
			  dim content as string = themessage.serialize()
			  if agent = "" then
			    agent = DefaultUserAgent
			  end
			  
			  self.setrequestheader "user-agent",agent
			  self.setpostcontent content,"text/xml"
			  self.lastmessage = themessage
			  self.secure = (left(self.rpcserver,5) = "https")
			  self.defaultport = self.rpcport
			  
			  SendBegin themessage
			  self.post(self.rpcserver)
			End Sub
		#tag EndMethod

		#tag Method, Flags = &h0
			Function SendSynchronousMessage(TheMessage As Message) As Variant
			  dim agent as string = useragent
			  dim xml as string
			  if agent = "" then
			    agent = DefaultUserAgent
			  end
			  
			  self.setrequestheader "user-agent",agent
			  self.setpostcontent themessage.serialize(),"text/xml"
			  self.lastmessage = themessage
			  self.secure = (left(self.rpcserver,5) = "https")
			  self.defaultport = self.rpcport
			  xml = self.post(self.rpcserver,10)
			  
			  dim response as variant = ProcessResponse(xml)
			  return response
			End Function
		#tag EndMethod


		#tag Hook, Flags = &h0
			Event Error(ErrorCode As Integer, ErrorMsg As String) As boolean
		#tag EndHook

		#tag Hook, Flags = &h0
			Event Open()
		#tag EndHook

		#tag Hook, Flags = &h0
			Event ResponseReceived(MethodName As String, Response As Variant)
		#tag EndHook

		#tag Hook, Flags = &h0
			Event ResponseReceiveProgress(IncomingMessage As XMLRPC.Message, BytesReceived As Double, BytesTotal As Double)
		#tag EndHook

		#tag Hook, Flags = &h0
			Event SendBegin(OutgoingMessage as XMLRPC.Message)
		#tag EndHook

		#tag Hook, Flags = &h0
			Event SendComplete(OutgoingMessage as XMLRPC.Message)
		#tag EndHook

		#tag Hook, Flags = &h0
			Event SendProgress(OutgoingMessage As XMLRPC.Message, BytesSent As Double, BytesTotal As Double)
		#tag EndHook

		#tag Hook, Flags = &h0
			Event UserAgent() As String
		#tag EndHook


		#tag Property, Flags = &h0
			RPCServer As String
		#tag EndProperty

		#tag Property, Flags = &h21
			Private LastMessage As Message
		#tag EndProperty

		#tag Property, Flags = &h21
			Private pLastError As Integer
		#tag EndProperty

		#tag Property, Flags = &h21
			Private pLastErrorString As String
		#tag EndProperty

		#tag Property, Flags = &h21
			Private OverridePort As Integer
		#tag EndProperty

		#tag ComputedProperty, Flags = &h0
			#tag Getter
				Get
				if me.overrideport = 0 then
				if self.secure then
				return kHTTPSPort
				else
				return kHTTPPort
				end
				else
				return me.overrideport
				end
				End Get
			#tag EndGetter
			#tag Setter
				Set
				me.overrideport = value
				End Set
			#tag EndSetter
			RPCPort As Integer
		#tag EndComputedProperty


		#tag Constant, Name = Version, Type = String, Dynamic = False, Default = \"2.0", Scope = Public
		#tag EndConstant

		#tag Constant, Name = kError_BadXML, Type = Double, Dynamic = False, Default = \"500", Scope = Public
		#tag EndConstant

		#tag Constant, Name = kError_ConnectionLost, Type = Double, Dynamic = False, Default = \"504", Scope = Public
		#tag EndConstant

		#tag Constant, Name = kError_HTTPError, Type = Double, Dynamic = False, Default = \"502", Scope = Public
		#tag EndConstant

		#tag Constant, Name = kError_NoResponse, Type = Double, Dynamic = False, Default = \"501", Scope = Public
		#tag EndConstant

		#tag Constant, Name = kError_ServerNotFound, Type = Double, Dynamic = False, Default = \"503", Scope = Public
		#tag EndConstant

		#tag Constant, Name = kError_SocketError, Type = Double, Dynamic = False, Default = \"505", Scope = Public
		#tag EndConstant

		#tag Constant, Name = kError_UnknownMethod, Type = Double, Dynamic = False, Default = \"506", Scope = Public
		#tag EndConstant


		#tag ViewBehavior
			#tag ViewProperty
				Name="Name"
				Visible=true
				Group="ID"
				InheritedFrom="SSLSocket"
			#tag EndViewProperty
			#tag ViewProperty
				Name="Index"
				Visible=true
				Group="ID"
				Type="Integer"
				InheritedFrom="SSLSocket"
			#tag EndViewProperty
			#tag ViewProperty
				Name="Super"
				Visible=true
				Group="ID"
				InheritedFrom="SSLSocket"
			#tag EndViewProperty
			#tag ViewProperty
				Name="Left"
				Visible=true
				Group="Position"
				InheritedFrom="SSLSocket"
			#tag EndViewProperty
			#tag ViewProperty
				Name="Top"
				Visible=true
				Group="Position"
				InheritedFrom="SSLSocket"
			#tag EndViewProperty
			#tag ViewProperty
				Name="ConnectionType"
				Visible=true
				Group="Position"
				EditorType="Integer"
				InheritedFrom="SSLSocket"
			#tag EndViewProperty
			#tag ViewProperty
				Name="CertificateFile"
				Visible=true
				Group="Position"
				EditorType="FolderItem"
				InheritedFrom="SSLSocket"
			#tag EndViewProperty
			#tag ViewProperty
				Name="CertificateRejectionFile"
				Visible=true
				Group="Position"
				EditorType="FolderItem"
				InheritedFrom="SSLSocket"
			#tag EndViewProperty
			#tag ViewProperty
				Name="Secure"
				Visible=true
				Group="Position"
				EditorType="Boolean"
				InheritedFrom="SSLSocket"
			#tag EndViewProperty
			#tag ViewProperty
				Name="CertificatePassword"
				Visible=true
				Group="Position"
				EditorType="String"
				InheritedFrom="SSLSocket"
			#tag EndViewProperty
			#tag ViewProperty
				Name="defaultPort"
				Group="Behavior"
				InitialValue="0"
				Type="integer"
				InheritedFrom="HTTPSecureSocket"
			#tag EndViewProperty
			#tag ViewProperty
				Name="httpProxyAddress"
				Group="Behavior"
				Type="string"
				EditorType="MultiLineEditor"
				InheritedFrom="HTTPSecureSocket"
			#tag EndViewProperty
			#tag ViewProperty
				Name="httpProxyPort"
				Group="Behavior"
				InitialValue="0"
				Type="integer"
				InheritedFrom="HTTPSecureSocket"
			#tag EndViewProperty
			#tag ViewProperty
				Name="yield"
				Group="Behavior"
				InitialValue="0"
				Type="boolean"
				InheritedFrom="HTTPSecureSocket"
			#tag EndViewProperty
			#tag ViewProperty
				Name="RPCServer"
				Group="Behavior"
				Type="String"
				EditorType="MultiLineEditor"
			#tag EndViewProperty
			#tag ViewProperty
				Name="RPCPort"
				Group="Behavior"
				InitialValue="0"
				Type="Integer"
			#tag EndViewProperty
		#tag EndViewBehavior
	End Class
	#tag EndClass
