#tag Module
Protected Module XMLRPC
	#tag Method, Flags = &h21
		Private Function Decode(item as xmlnode) As Variant
		  if item = nil then
		    return ""
		  end
		  
		  dim datatype as string = item.name
		  dim child as xmlnode = item.child(0)
		  dim i as integer
		  dim dict as dictionary
		  dim arr() as variant
		  dim stamp as date
		  
		  select case (datatype)
		  case kDataType_Array
		    for i = 0 to child.childcount - 1
		      arr.append decode(child.child(i).child(0))
		    next
		    return arr
		  case kDataType_Binary
		    if child = nil then
		      return ""
		    else
		      return decodebase64(child.value)
		    end
		  case kDataType_Boolean
		    if child = nil then
		      return false
		    end
		    if child.value = "1" then
		      return true
		    else
		      return false
		    end
		  case kDataType_Date
		    if child <> nil then
		      stamp = parseiso8601(child.value)
		    else
		      stamp = new date
		    end
		    return stamp
		  case kDataType_Dictionary
		    dict = new dictionary
		    if child <> nil then
		      for i = 0 to item.childcount - 1
		        child = item.child(i)
		        dict.value(child.child(0).child(0).value) = decode(child.child(1).child(0))
		      next
		    end
		    return dict
		  case kDataType_Double, kDataType_Integer, kDataType_Integer4
		    if child = nil then
		      return 0
		    else
		      return val(child.value)
		    end
		  case kDataType_String
		    if child = nil then
		      return ""
		    else
		      return child.value
		    end
		  end
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Function ParseISO8601(Input As String) As Date
		  dim r as regex
		  dim m as regexmatch
		  dim d as new date
		  
		  //19980717T14:08:55
		  // (\d\d\d\d)(\d\d)(\d\d)T(\d\d)\:(\d\d)\:(\d\d)
		  
		  r = new regex
		  r.searchpattern = "^(\d\d\d\d)\-?(\d\d)\-?(\d\d)T(\d\d)\:(\d\d)\:(\d\d)"
		  m = r.search(input)
		  if m <> nil then
		    d.year = val(m.subexpressionstring(1))
		    d.month = val(m.subexpressionstring(2))
		    d.day = val(m.subexpressionstring(3))
		    d.hour = val(m.subexpressionstring(4))
		    d.minute = val(m.subexpressionstring(5))
		    d.second = val(m.subexpressionstring(6))
		  end
		  
		  return d
		End Function
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Function Scrap() As XMLDocument
		  static pscrap as xmldocument
		  if pscrap = nil then
		    pscrap = new xmldocument
		  end
		  return pscrap
		End Function
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Function Encode(Param As Pair, xmldoc as xmldocument) As xmlelement
		  dim type as string = param.left
		  dim value as variant = param.right
		  dim node as xmlelement = xmldoc.createelement(type)
		  dim s as string
		  dim i as integer
		  
		  select case type
		  case kDataType_Array
		    dim data as xmlelement = xmldoc.createelement("data")
		    dim xmlvalue as xmlelement
		    dim arr() as variant = value
		    for i = 0 to ubound(arr)
		      xmlvalue = xmldoc.createelement("value")
		      xmlvalue.appendchild encode(varianttopair(arr(i)),xmldoc)
		      data.appendchild xmlvalue
		    next
		    node.appendchild data
		  case kDataType_Boolean
		    if value.booleanvalue then
		      node.appendchild xmldoc.createtextnode("1")
		    else
		      node.appendchild xmldoc.createtextnode("0")
		    end
		  case kDataType_Date
		    s = zeropad(value.datevalue.year,4)
		    s = s + zeropad(value.datevalue.month)
		    s = s + zeropad(value.datevalue.day)
		    s = s + "T"
		    s = s + zeropad(value.datevalue.hour) + ":"
		    s = s + zeropad(value.datevalue.minute) + ":"
		    s = s + zeropad(value.datevalue.second)
		    node.appendchild xmldoc.createtextnode(s)
		  case kDataType_Dictionary
		    dim member,name,xmlvalue as xmlelement
		    dim dict as dictionary = dictionary(value.objectvalue)
		    for i = 0 to dict.count - 1
		      member = xmldoc.createelement("member")
		      name = xmldoc.createelement("name")
		      name.appendchild xmldoc.createtextnode(dict.key(i))
		      xmlvalue = xmldoc.createelement("value")
		      xmlvalue.appendchild encode(varianttopair(dict.value(dict.key(i))),xmldoc)
		      member.appendchild name
		      member.appendchild xmlvalue
		      node.appendchild member
		    next
		  case kDataType_Double
		    node.appendchild xmldoc.createtextnode(format(value.doublevalue,"-#.####################"))
		  case kDataType_Integer, kDataType_Integer4
		    node.appendchild xmldoc.createtextnode(format(value.integervalue,"-#"))
		  case kDataType_String, kDataType_Binary
		    node.appendchild xmldoc.createtextnode(value.stringvalue)
		  end
		  
		  return node
		End Function
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Function ZeroPad(Value As Integer, MinLength As Integer = 2) As string
		  dim i as integer
		  dim s as string
		  for i = 1 to minlength - len(str(value))
		    s = s + "0"
		  next
		  return s + str(value)
		End Function
	#tag EndMethod

	#tag Method, Flags = &h1
		Protected Function GuessType(Value As Variant) As String
		  dim type as integer = value.type
		  select case type
		  case (variant.typearray or value.arrayelementtype)
		    return kDataType_Array
		  case variant.typeboolean
		    return kDataType_Boolean
		  case variant.typedate
		    return kDataType_Date
		  case variant.typedouble
		    return kDataType_Double
		  case variant.typeinteger
		    return kDataType_Integer
		  case variant.typeobject
		    if value isa dictionary then
		      return kDataType_Dictionary
		    else
		      return ""
		    end
		  case variant.typestring
		    return kDataType_String
		  else
		    return ""
		  end
		End Function
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Function VariantToPair(Input As Variant) As Pair
		  if input isa pair then
		    return input
		  else
		    return GuessType(input) : input
		  end
		End Function
	#tag EndMethod


	#tag Constant, Name = kDataType_Array, Type = String, Dynamic = False, Default = \"array", Scope = Private
	#tag EndConstant

	#tag Constant, Name = kDataType_Binary, Type = String, Dynamic = False, Default = \"base64", Scope = Private
	#tag EndConstant

	#tag Constant, Name = kDataType_Boolean, Type = String, Dynamic = False, Default = \"boolean", Scope = Private
	#tag EndConstant

	#tag Constant, Name = kDataType_Date, Type = String, Dynamic = False, Default = \"dateTime.iso8601", Scope = Private
	#tag EndConstant

	#tag Constant, Name = kDataType_Dictionary, Type = String, Dynamic = False, Default = \"struct", Scope = Private
	#tag EndConstant

	#tag Constant, Name = kDataType_Double, Type = String, Dynamic = False, Default = \"double", Scope = Private
	#tag EndConstant

	#tag Constant, Name = kDataType_Integer, Type = String, Dynamic = False, Default = \"int", Scope = Private
	#tag EndConstant

	#tag Constant, Name = kDataType_Integer4, Type = String, Dynamic = False, Default = \"i4", Scope = Private
	#tag EndConstant

	#tag Constant, Name = kDataType_String, Type = String, Dynamic = False, Default = \"string", Scope = Private
	#tag EndConstant

	#tag Constant, Name = kHTTPPort, Type = Double, Dynamic = False, Default = \"80", Scope = Protected
	#tag EndConstant

	#tag Constant, Name = kHTTPSPort, Type = Double, Dynamic = False, Default = \"443", Scope = Protected
	#tag EndConstant


	#tag ViewBehavior
		#tag ViewProperty
			Name="Name"
			Visible=true
			Group="ID"
			InheritedFrom="Object"
		#tag EndViewProperty
		#tag ViewProperty
			Name="Index"
			Visible=true
			Group="ID"
			InitialValue="2147483648"
			InheritedFrom="Object"
		#tag EndViewProperty
		#tag ViewProperty
			Name="Super"
			Visible=true
			Group="ID"
			InheritedFrom="Object"
		#tag EndViewProperty
		#tag ViewProperty
			Name="Left"
			Visible=true
			Group="Position"
			InitialValue="0"
			InheritedFrom="Object"
		#tag EndViewProperty
		#tag ViewProperty
			Name="Top"
			Visible=true
			Group="Position"
			InitialValue="0"
			InheritedFrom="Object"
		#tag EndViewProperty
	#tag EndViewBehavior
End Module
#tag EndModule
